@@ -25,23 +25,38 @@ second.principle = function(x, cluster = NULL, mb, whitelist, blacklist,
     # 3.2 sort them in p-value order.
     vs = vs[order(vs[,"max_a"], decreasing = FALSE),]
     # 3.3 apply.
-    arcs = vstruct.apply(arcs = arcs, vs = vs, strict = strict, debug = debug)
+    arcs = vstruct.apply(arcs = arcs, vs = vs, nodes = nodes, 
+             strict = strict, debug = debug)
 
   }#THEN
 
   # 4. [Remove Cycles] and 5. [Reverse Edges]
   # thou shalt not create cycles in the graph, it's acyclic!
-  arcs = orient.edges(arcs = arcs, nodes = nodes,
-           whitelist = whitelist, blacklist = blacklist,
-           cluster = cluster, debug = debug)
+  arcs = orient.edges(arcs = arcs, nodes = nodes, whitelist = whitelist, 
+           blacklist = blacklist, cluster = cluster, debug = debug,
+           pass = 1)
 
   # 6. [Propagate Directions]
-  arcs = propagate.directions(arcs = arcs, nodes = nodes, debug = debug)
+  arcs = orient.edges(arcs = arcs, nodes = nodes, whitelist = whitelist, 
+           blacklist = blacklist, cluster = cluster, debug = debug,
+           pass = 2)
 
-  # arcs whitelisted in one direction (i.e. a -> b but not b -> a) are
-  # never checked for cycles, do it now.
-  if (!is.acyclic(arcs = arcs, nodes = nodes))
-    stop("the graph contains cycles, possibly because of whitelisted nodes.")
+  # do a last sanity check, just in case.
+  if (!is.acyclic.backend(arcs = arcs, nodes = nodes, directed = FALSE)) {
+
+    if (strict)
+      stop("there are still cycles in the graph, aborting.")
+    else
+      warning("there are still cycles in the graph, fixing.")
+
+    arcs = orient.edges(arcs = arcs, nodes = nodes, whitelist = whitelist,
+             blacklist = blacklist, cluster = cluster, debug = debug,
+             pass = 3)
+
+    if (!is.acyclic.backend(arcs = arcs, nodes = nodes, directed = FALSE))
+      stop("the graph still contains cycles.")
+
+  }#THEN
 
   # save the status of the learning algorithm.
   learning = list(whitelist = whitelist, blacklist = blacklist,
@@ -58,56 +73,6 @@ second.principle = function(x, cluster = NULL, mb, whitelist, blacklist,
 
 }#SECOND.PRINCIPLE
 
-# propagate directions to the undirected arcs.
-propagate.directions = function(arcs, nodes, debug) {
-
-  # build the adjacency matrix.
-  amat = arcs2amat(arcs, nodes)
-  # ignore directed arcs.
-  undirected.arcs = arcs[which.undirected(arcs),]
-
-  if (debug) {
-
-    cat("----------------------------------------------------------------\n")
-    cat("* propagating directions for the following undirected arcs:\n")
-
-  }#THEN
-
-  if (nrow(undirected.arcs) == 0) {
-
-    if (debug) cat("  > nothing to do.\n")
-
-  }#THEN
-  else apply(undirected.arcs, 1, function(arc) {
-
-    # if there is a path from arc[1] to arc[2] besides arc[1] -> arc[2] ...
-    if (has.path(arc[1], arc[2], nodes, amat, exclude.direct = TRUE)) {
-
-      # ... set the direction to arc[1] -> arc[2].
-      assign("arcs", set.arc.direction(arc[1], arc[2], arcs),
-        envir = sys.frame(-2))
-
-      if (debug) {
-
-        cat("  > there's a path from", arc[1], "to", arc[2], ".\n")
-        cat("    > removing", arc[2], "->", arc[1], ".\n")
-
-      }#THEN
-
-    }#THEN
-    else {
-
-      if (debug)
-        cat("  > no path from", arc[1], "to", arc[2], ".\n")
-
-    }#ELSE
-
-  })
-
-  arcs
-
-}#PROPAGATE.DIRECTIONS
-
 # build the neighbourhood of a node from the markov blanket.
 neighbour = function(x, mb, data, alpha, B = NULL, whitelist, blacklist,
   backtracking = NULL, test, debug) {
@@ -306,9 +271,10 @@ vstruct.detect = function(nodes, arcs, mb, data, alpha, B = NULL, test, debug) {
             if (debug)
               cat("    > testing", y, "vs", z, "given", c(dsep.subsets[s,], x), "(", a, ")\n")
             max_a = max(a, max_a)
-            if ((a > alpha) && debug) {
+            if (a > alpha) {
 
-              cat("    >", y, "and", z, "are independent given '", c(dsep.subsets[s,], x), "' (", a, ")\n")
+              if (debug)
+                cat("    >", y, "and", z, "are independent given '", c(dsep.subsets[s,], x), "' (", a, ")\n")
               break
 
             }#THEN
@@ -353,35 +319,19 @@ vstruct.detect = function(nodes, arcs, mb, data, alpha, B = NULL, test, debug) {
 }#VSTRUCT.DETECT
 
 # apply v-structures to a graph.
-vstruct.apply = function(arcs, vs, strict, debug) {
+vstruct.apply = function(arcs, vs, nodes, strict, debug) {
 
   if (debug)
     cat("----------------------------------------------------------------\n")
 
   apply(vs, 1, function(v) {
 
-    # check the arcs of the v-strustures are still there.
-    if(is.listed(arcs, v[c("y", "x")]) &&
-       is.listed(arcs, v[c("z", "x")])) {
-
-      if (debug)
-        cat("* applying v-structure", v["y"], "->", v["x"], "<-", v["z"],
-              "(", v["max_a"], ")\n")
-
-      assign("arcs", set.arc.direction(v["y"], v["x"], arcs),
-        envir = sys.frame(-2))
-      assign("arcs", set.arc.direction(v["z"], v["x"], arcs),
-        envir = sys.frame(-2))
-
-    }#THEN
-    else {
+    if (!(is.listed(arcs, v[c("y", "x")]) && is.listed(arcs, v[c("z", "x")]))) {
 
       if (debug) {
 
         cat("* not applying v-structure", v["y"], "->", v["x"], "<-", v["z"],
               "(", v["max_a"], ")\n")
-        cat("  > adding arc", v["y"], "->", v["z"], "\n")
-        cat("  > adding arc", v["z"], "->", v["y"], "\n")
 
       }#THEN
 
@@ -394,7 +344,19 @@ vstruct.apply = function(arcs, vs, strict, debug) {
           "is not applicable, because one or both arcs are oriented",
           "in the opposite direction."))
 
-    }#ELSE
+      return(NULL)
+
+    }#THEN
+
+    # save the updates arc set.
+    temp = set.arc.direction(v["y"], v["x"], arcs)
+    temp = set.arc.direction(v["z"], v["x"], temp)
+
+    if (debug)
+      cat("* applying v-structure", v["y"], "->", v["x"], "<-", v["z"],
+            "(", v["max_a"], ")\n")
+
+    assign("arcs", temp, envir = sys.frame(-2))
 
   })
 
@@ -403,7 +365,8 @@ vstruct.apply = function(arcs, vs, strict, debug) {
 }#VSTRUCT.APPLY
 
 # remove arcs from the graph to make it acyclic.
-orient.edges = function(arcs, nodes, whitelist, blacklist, cluster, debug) {
+orient.edges = function(arcs, nodes, whitelist, blacklist, pass, cluster, 
+    debug) {
 
   to.be.reversed = character(0)
   narcs = nrow(arcs)
@@ -412,10 +375,32 @@ orient.edges = function(arcs, nodes, whitelist, blacklist, cluster, debug) {
   # remove undirected arcs from the whitelist; their direction is
   # left to the algorithm to choose.
   if (!is.null(whitelist) && (nrow(whitelist) > 0))
-    whitelist = whitelist[!which.undirected(whitelist),, drop = FALSE]
+    whitelist = whitelist[which.directed(whitelist),, drop = FALSE]
+
+  # the first pass considers only directed arcs; drop the undirected ones.
+  if (pass == 1) {
+
+    u = arcs[which.undirected(arcs),, drop = FALSE]
+
+    arcs = arcs[which.directed(arcs),, drop = FALSE]
+
+  }#THEN
 
   repeat {
 
+    # the second pass has the main purpose to remove the propagate the directions
+    # of the directed arcs; do not try to drop/reverse them again.
+    if (pass == 2) {
+  
+      d = arcs[which.directed(arcs),, drop = FALSE]
+  
+      if (!is.null(whitelist))
+        whitelist = arcs.rbind(whitelist, d)
+      else
+        whitelist = d
+  
+    }#THEN
+  
     # cannot remove or reverse more arcs than you have.
     if (n > narcs)
       stop("too many iterations, probably would have gone on forever.")
@@ -427,7 +412,7 @@ orient.edges = function(arcs, nodes, whitelist, blacklist, cluster, debug) {
     if (debug) {
 
       cat("----------------------------------------------------------------\n")
-      cat("* detecting cycles ...\n")
+      cat("*", switch(pass, "detecting cycles", "propagating directions", "fixing remaining cycles"), "...\n")
       cat("  > ignored nodes:", length(nodes) - length(in.cycles), "/", length(nodes), "\n")
       cat("  > ignored arcs:", length(which(!to.be.checked)), "/", nrow(arcs), "\n")
       print(arcs[!to.be.checked, , drop = FALSE])
@@ -439,7 +424,7 @@ orient.edges = function(arcs, nodes, whitelist, blacklist, cluster, debug) {
     cycles = cycle.counter(arcs = arcs[to.be.checked,, drop = FALSE],
               nodes = in.cycles, cluster = cluster)
 
-    # do not check arcs whitelisted directed arcs.
+    # do not check whitelisted directed arcs.
     cycles = cycles[!which.listed(cycles[, 1:2], whitelist),, drop = FALSE]
 
     if (debug && (nrow(cycles) > 0)) print(cycles)
@@ -447,25 +432,101 @@ orient.edges = function(arcs, nodes, whitelist, blacklist, cluster, debug) {
     # if there are no more arcs to check, break.
     if (nrow(cycles) == 0) break
 
-    # remove the arc belonging to the highest number of cycles.
-    arcs = arcs[!is.row.equal(arcs, cycles[1, c("from", "to")]),, drop = FALSE]
+    # if an arc belongs to a completely directed cycle but do it reverse does
+    # not, drop it from the cycles data frame (this makes the second pass compliant
+    # with Margaritis' algorithm specification). 
+    if (pass == 2) {
+
+        amatd = arcs2amat(arcs[which.directed(arcs),, drop = FALSE], nodes)
+        candidates = which.undirected(as.matrix(cycles[, 1:2, drop = FALSE]))
+
+        for (i in which(candidates)) {
+
+          res = !has.path(cycles[i, 2], cycles[i, 1], nodes, amatd, exclude.direct = TRUE) && 
+                 has.path(cycles[i, 1], cycles[i, 2], nodes, amatd, exclude.direct = TRUE)
+          candidates[i] = res
+
+        }#FOR
+ 
+        cycles = cycles[!candidates,, drop = FALSE]
+
+    }#THEN
+
+    if (pass != 1) {
+
+      changed = FALSE
+
+      # check which arcs belong to the maximum number of cycles in both 
+      # directions and ignore them; removing any of them does not make
+      # sense.
+      max.cycles = cycles[1, 3]
+      which.max.cycles = (cycles[, 3] == max.cycles)
+      candidates = which.undirected(as.matrix(cycles[which.max.cycles, 1:2, drop = FALSE]))
+
+      if ((length(which(candidates)) > 0) & (nrow(cycles[!candidates,, drop = FALSE]) > 0)) {
+
+        cycles = cycles[!candidates,, drop = FALSE]
+        changed = TRUE
+
+      }#THEN
+
+      # if there is more than one candidate for removal, remove the one whose
+      # reverse belongs to the least number of cycles.
+      max.cycles = cycles[1, 3]
+      which.max.cycles = (cycles[, 3] == max.cycles) & which.undirected(as.matrix(cycles[, 1:2]))
+
+      if (length(which(which.max.cycles)) > 1) {
+
+        # get the reverse of the arcs belonging to maximum number of cycles.
+        candidates = as.matrix(cycles[which.max.cycles, 2:1, drop = FALSE])
+        # get their indexes in the arc set.
+        candidates = which.listed(as.matrix(cycles[, 1:2, drop = FALSE]), candidates)
+        # order the arcs according to the number of cycles their reverses
+        # belong to.
+        cycles2 = cycles[candidates,, drop = FALSE]
+        cycle.order = order(cycles2[, 3, drop = FALSE], decreasing = FALSE)
+        cycles[which.max.cycles,] = cycles2[cycle.order,, drop = FALSE]
+        changed = TRUE
+
+      }#THEN
+
+      if (debug && changed) {
+
+        cat("  > reordered arcs (belonging to", max.cycles, "cycles each).\n")
+        which.max.cycles = (cycles[, 3] == max.cycles)
+        print(cycles[which.max.cycles,, drop = FALSE])
+
+      }#THEN
+
+    }#THEN
 
     # if the arc was already directed and its reverse is not blacklisted,
     # store it for future reversal.
     if (is.directed(cycles[1, c("from", "to")], arcs) &&
         !is.blacklisted(blacklist, cycles[1, c("to", "from")])) {
 
+      if (debug)
+        cat("  > arc", cycles[1, "from"], "->", cycles[1, "to"],
+          "will be reversed (", cycles[1, "cycles"], " cycles ).\n")
+
       to.be.reversed = c(to.be.reversed, cycles[1, "from"], cycles[1, "to"])
 
     }#THEN
+    else {
+
+      if (debug)
+        cat("  > arc", cycles[1, "from"], "->", cycles[1, "to"],
+          "will be removed (", cycles[1, "cycles"], " cycles ).\n")
+
+    }#THEN
+
+    # remove the arc belonging to the highest number of cycles.
+    arcs = arcs[!is.row.equal(arcs, cycles[1, c("from", "to")]),, drop = FALSE]
 
     if (debug) {
 
-      cat("  > arc", cycles[1, "from"], "->", cycles[1, "to"],
-        "will be removed (", cycles[1, "cycles"], " cycles ).\n")
-
       cat("  > arcs scheduled for reversal are:\n")
-      print(matrix(to.be.reversed, ncol = 2, byrow = TRUE, 
+      print(matrix(to.be.reversed, ncol = 2, byrow = TRUE,
               dimnames = list(NULL, c("from", "to"))))
 
     }#THEN
@@ -476,8 +537,11 @@ orient.edges = function(arcs, nodes, whitelist, blacklist, cluster, debug) {
 
   # add removed arcs, reversed and return.
   to.be.reversed = matrix(to.be.reversed, ncol = 2, byrow = TRUE)
-  matrix(c(arcs[, 1], to.be.reversed[, 2], arcs[, 2], to.be.reversed[, 1]), 
-    ncol = 2, dimnames = list(NULL, c("from", "to")), byrow = FALSE)
+
+  if (pass == 1)
+    return(arcs.rbind(arcs.rbind(u, arcs), to.be.reversed, reverse2 = TRUE))
+  else
+    return(arcs.rbind(arcs, to.be.reversed, reverse2 = TRUE))
 
 }#ORIENT.EDGES
 
@@ -512,108 +576,6 @@ cycle.counter = function(arcs, nodes, cluster, debug = FALSE) {
 
 }#CYCLE.COUNTER
 
-# test undirected arcs in both direction to infer their orientation.
-set.directions = function(arcs, data, test, alpha, B = NULL, cluster, debug) {
-
-  if (debug) {
-
-    cat("----------------------------------------------------------------\n")
-    cat("* infer arc direction.\n")
-
-  }#THEN
-
-  # get the undirected arcs.
-  undirected.arcs = arcs[which.undirected(arcs), , drop = FALSE]
-
-  if (nrow(undirected.arcs) == 0)
-    return(arcs)
-
-  if (is.null(cluster)) {
-
-    a = apply(undirected.arcs, 1, function(arc) {
-
-      # you can't help but notice nodes connecetd by undirected arcs are
-      # included, too? wonder why?
-      # because if they, too, are parents of the node to be tested
-      # they _do_ belong there; if they are not, the node distribution
-      # does not depend on them so they are largely irrelevant.
-      parents = parents.backend(arcs, arc[2], TRUE)
-      conditional.test(arc[1], arc[2],
-          parents[parents != arc[1]],
-          data = data, test = test, B = B)
-
-    })
-
-  }#THEN
-  else {
-
-     a = parApply(cluster, undirected.arcs, 1, function(arc, data, test) {
-
-      # you can't help but notice nodes connecetd by undirected arcs are
-      # included, too? wonder why?
-      # because if they, too, are parents of the node to be tested
-      # they _do_ belong there; if they are not, the node distribution
-      # does not depend on them so they are largely irrelevant.
-      parents = parents.backend(arcs, arc[2], TRUE)
-      conditional.test(arc[1], arc[2],
-          parents[parents != arc[1]],
-          data = data, test = test, B = B)
-
-    }, data = data, test = test, B = B)
-
-  }#ELSE
-
-  # build the lookup table.
-  lookup.table = cbind(undirected.arcs, a)
-
-  # sort arcs by increasing p-value.
-  lookup.table = lookup.table[order(as.numeric(a)), ]
-
-  repeat {
-
-    # no more arcs, breaking the loop.
-    if(nrow(lookup.table) == 0) break
-
-    # get the most likely arc.
-    da.best = lookup.table[1, ]
-
-    if (debug) {
-
-      cat("  > the results of the conditional independence tests are:\n")
-      print(lookup.table)
-
-    }#THEN
-
-    # if both arcs have the same p-value, do not remove them from the arc set.
-    if (da.best[3] != lookup.table[is.row.equal(lookup.table[, 1:2], da.best[c(2, 1)]), 3]) {
-
-      # remove its reverse from the arc set.
-      arcs = set.arc.direction(da.best[1], da.best[2], arcs)
-
-      if (debug) {
-
-        cat("  > ", da.best[1], "->", da.best[2], "has the lowest alpha (", da.best[3], ").\n")
-        cat("  > ", da.best[2], "->", da.best[1], "will be removed.\n")
-
-      }#THEN
-
-    }#THEN
-    else {
-
-      if (debug) cat("  > it's a draw, no arc will be removed.\n")
-
-    }#ELSE
-
-    # remove both of them from the lookup table
-    lookup.table = lookup.table[!is.row.equal(lookup.table[, 1:2], da.best[1:2]) &
-                                !is.row.equal(lookup.table[, 1:2], da.best[c(2, 1)]), ]
-
-  }#REPEAT
-
-  arcs
-
-}#SET.DIRECTIONS
-
 # emergency measures for markov blanket and neighbourhood recovery.
 bn.recovery = function(bn, nodes, strict, mb = FALSE, debug) {
 
