@@ -13,6 +13,13 @@
     cat("Package Rgraphviz", 
       ifelse(graphviz.enabled, "loaded successfully.\n", "not loaded.\n"))
 
+    # load lattice and set the corresponding flag.
+    if ("lattice" %in% rownames(installed.packages()))
+      lattice.enabled <<- require(lattice)
+
+    cat("Package lattice", 
+      ifelse(lattice.enabled, "loaded successfully.\n", "not loaded.\n"))
+
     # restore the original warning level.
     options("warn" = warning.level)
 
